local gm = require("GeoMan")

local win_w, win_h = love.graphics.getDimensions()

local lg = love.graphics
local lk = love.keyboard

local timerKey = 0

-- BEEP SOUND COLLISION --

local rate = 44100
local length = 1/32
local tone = 440.0
local p = math.floor(rate/tone)
local soundData = love.sound.newSoundData(math.floor(length*rate), rate, 16, 1)
for i=0, soundData:getSampleCount() - 1 do
	soundData:setSample(i, i%p<p/2 and 1 or -1)
end
local source = love.audio.newSource(soundData)
local function beep() source:play() end


-- PLAYER --

local player = {
    x = (win_w-50)/2,
    y = (win_h-50)/2,
    larger = 50,
    speed = 300
}

player.verts = gm.hexagon(player.x, player.y, player.larger)

function player:update(dt)

    local vx, vy = 0, 0

    if lk.isDown("up") then vy = vy - 1 end
    if lk.isDown("down") then vy = vy + 1 end
    if lk.isDown("left") then vx = vx - 1 end
    if lk.isDown("right") then vx = vx + 1 end

    local len = math.sqrt(vx*vx+vy*vy)

    if len > 0 then

        vx = vx~=0 and vx/len or vx
        vy = vy~=0 and vy/len or vy

        self.x = self.x + vx * self.speed * dt
        self.y = self.y + vy * self.speed * dt

        gm.setPolyPosition(self.verts, self.x, self.y)

    end

end

function player:draw()
    lg.polygon("fill", self.verts)
end


-- ENNEMIES --

local ennemies = {
    larger=10,
    list = {},
    max = 10
}

function ennemies:update(dt)

    if timerKey == 0 then

        if lk.isDown("space") then
            self.max, timerKey = self.max + 1, .1
        elseif self.max > 1 and lk.isDown("lctrl") then
            self.max, timerKey = self.max - 1, .1
            self.list[#self.list] = nil
        end

    else
        timerKey = timerKey > 0 and timerKey - dt or 0
    end

    while #self.list ~= self.max do

        self.list[#self.list+1] = {
            x = math.random(0,win_w-self.larger),
            y = math.random(0,win_h-self.larger),
            vx = math.random() > .5 and 1 or -1,
            vy = math.random() > .5 and 1 or -1
        }

        local e = self.list[#self.list]

        e.verts = gm.hexagon(e.x, e.y, self.larger)

    end

    for k, v in pairs(self.list) do

        v.x = v.x + v.vx * 100 * dt
        v.y = v.y + v.vy * 100 * dt

        gm.setPolyPosition(v.verts, v.x, v.y)

        local collide
        if gm.isPolyInPoly(v.verts, player.verts) then
            beep(); collide = true
        end

        if collide -- collision or out of screen
        or ((v.x < 0 or v.x+self.larger > win_w)
        and (v.y < 0 or v.y+self.larger > win_h))
        then table.remove(self.list, k) end

    end

end

function ennemies:draw()
    for _, v in pairs(self.list) do
        lg.polygon("fill", v.verts)
    end
end


-- MAIN LOOP --

function love.update(dt)

    player:update(dt)
    ennemies:update(dt)

end


function love.draw()

    lg.setColor(1,1,0)
    lg.print("FPS: "..love.timer.getFPS())
    lg.print("Number MAX of ennemies: "..ennemies.max,0,win_h-32)
    lg.print("SPACE for more & LCTRL for less. ",0,win_h-16)

    lg.setColor(0,1,0)
    player:draw()

    lg.setColor(1,0,0)
    ennemies:draw()

end

